@@ -1,5 +1,5 @@
 /* krb5/oid.c --- Definition of static Kerberos 5 GSS-API OIDs.
- * Copyright (C) 2003, 2004, 2005, 2006, 2007  Simon Josefsson
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
  *
  * This file is part of the Generic Security Service (GSS).
  *
@@ -24,14 +24,6 @@
 #include "k5internal.h"
 
 /*
- * To support ongoing experimentation, testing, and evolution of the
- * specification, the Kerberos V5 GSS-API mechanism as defined in this
- * and any successor memos will be identified with the following
- * Object Identifier, as defined in RFC-1510, until the specification
- * is advanced to the level of Proposed Standard RFC:
- *
- * {iso(1), org(3), dod(5), internet(1), security(5), kerberosv5(2)}
- *
  * Upon advancement to the level of Proposed Standard RFC, the
  * Kerberos V5 GSS-API mechanism will be identified by an Object
  * Identifier having the value:
@@ -77,6 +69,17 @@ gss_OID_desc GSS_KRB5_NT_PRINCIPAL_NAME_static = {
 };
 gss_OID GSS_KRB5_NT_PRINCIPAL_NAME = &GSS_KRB5_NT_PRINCIPAL_NAME_static;
 
+/*
+ * This name form shall be represented by the Object Identifier
+ * {iso(1) member-body(2) United States(840) mit(113554) infosys(1)
+ * gssapi(2) generic(1) machine_uid_name(2)}.  The recommended
+ * symbolic name for this type is "GSS_KRB5_NT_MACHINE_UID_NAME".
+ */
+gss_OID_desc GSS_KRB5_NT_MACHINE_UID_NAME_static = {
+  10, (void *) "\x2a\x86\x48\x86\xf7\x12\x01\x02\x02\x02"
+};
+gss_OID GSS_KRB5_NT_MACHINE_UID_NAME = &GSS_KRB5_NT_MACHINE_UID_NAME_static;
+
 /*
  * This name form shall be represented by the Object Identifier
  * {iso(1) member-body(2) United States(840) mit(113554) infosys(1)
